==> buildbot_4.0.2.orig.tar.gz: buildbot-4.0.2/master/buildbot/test/unit/schedulers/test_timed_NightlyTriggerable.py <==
# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members

from twisted.internet import task
from twisted.trial import unittest

from buildbot.process import properties
from buildbot.schedulers import timed
from buildbot.test import fakedb
from buildbot.test.reactor import TestReactorMixin
from buildbot.test.util import scheduler


class NightlyTriggerable(scheduler.SchedulerMixin, TestReactorMixin, unittest.TestCase):
    SCHEDULERID = 327
    OBJECTID = 1327

    def makeScheduler(self, firstBuildDuration=0, **kwargs):
        sched = self.attachScheduler(
            timed.NightlyTriggerable(**kwargs),
            self.OBJECTID,
            self.SCHEDULERID,
            overrideBuildsetMethods=True,
            createBuilderDB=True,
        )

        # add a Clock to help checking timing issues
        self.clock = sched._reactor = task.Clock()

        return sched

    def setUp(self):
        self.setup_test_reactor()
        self.setUpScheduler()

    def tearDown(self):
        self.tearDownScheduler()

    # utilities

    def assertBuildsetAdded(self, sourcestamps=None, properties=None):
        if sourcestamps is None:
            sourcestamps = {}
        if properties is None:
            properties = {}
        properties['scheduler'] = ('test', 'Scheduler')
        self.assertEqual(
            self.addBuildsetCalls,
            [
                (
                    'addBuildsetForSourceStampsWithDefaults',
                    {
                        "builderNames": None,  # uses the default
                        "priority": None,
                        "properties": properties,
                        "reason": "The NightlyTriggerable scheduler named 'test' triggered this build",
                        "sourcestamps": sourcestamps,
                        "waited_for": False,
                    },
                ),
            ],
        )
        self.addBuildsetCalls = []

    def assertNoBuildsetAdded(self):
        self.assertEqual(self.addBuildsetCalls, [])

    # tests

    def test_constructor_no_reason(self):
        sched = self.makeScheduler(name='test', builderNames=['test'])
        self.assertEqual(
            sched.reason, "The NightlyTriggerable scheduler named 'test' triggered this build"
        )

    def test_constructor_reason(self):
        sched = self.makeScheduler(name='test', builderNames=['test'], reason="hourlytriggerable")
        self.assertEqual(sched.reason, "hourlytriggerable")

    def test_constructor_month(self):
        sched = self.makeScheduler(name='test', builderNames=['test'], month='1')
        self.assertEqual(sched.month, "1")

    def test_timer_noBuilds(self):
        sched = self.makeScheduler(name='test', builderNames=['test'], minute=[5])

        sched.activate()
        self.clock.advance(60 * 60)  # Run for 1h

        self.assertEqual(self.addBuildsetCalls, [])

    def test_timer_oneTrigger(self):
        sched = self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        sched.activate()

        sched.trigger(
            False,
            [
                {
                    "revision": 'myrev',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "codebase": 'cb',
                },
            ],
            set_props=None,
        )

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev',
                },
            ]
        )

    def test_timer_twoTriggers(self):
        sched = self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        sched.activate()

        sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev1',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                }
            ],
            set_props=None,
        )
        sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev2',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                }
            ],
            set_props=None,
        )

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    # builds the second trigger's revision
                    "revision": 'myrev2',
                },
            ]
        )

    def test_timer_oneTrigger_then_noBuild(self):
        sched = self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        sched.activate()

        sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                }
            ],
            set_props=None,
        )

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev',
                },
            ]
        )

        self.clock.advance(60 * 60)  # Run for 1h

        # no trigger, so the second did not build
        self.assertNoBuildsetAdded()

    def test_timer_oneTriggers_then_oneTrigger(self):
        sched = self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        sched.activate()

        sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev1',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                }
            ],
            set_props=None,
        )

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev1',
                },
            ]
        )

        sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev2',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                }
            ],
            set_props=None,
        )

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev2',
                },
            ]
        )

    def test_savedTrigger(self):
        sched = self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        value_json = (
            '[ [ {"codebase": "cb", "project": "p", "repository": "r", '
            '"branch": "br", "revision": "myrev"} ], {}, null, null ]'
        )

        self.db.insert_test_data([
            fakedb.Object(id=self.SCHEDULERID, name='test', class_name='NightlyTriggerable'),
            fakedb.ObjectState(
                objectid=self.SCHEDULERID, name='lastTrigger', value_json=value_json
            ),
        ])

        sched.activate()

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev',
                },
            ]
        )

    def test_savedTrigger_dict(self):
        sched = self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        value_json = (
            '[ { "cb": {"codebase": "cb", "project": "p", "repository": "r", '
            '"branch": "br", "revision": "myrev"} }, {}, null, null ]'
        )
        self.db.insert_test_data([
            fakedb.Object(id=self.SCHEDULERID, name='test', class_name='NightlyTriggerable'),
            fakedb.ObjectState(
                objectid=self.SCHEDULERID, name='lastTrigger', value_json=value_json
            ),
        ])

        sched.activate()

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev',
                },
            ]
        )

    def test_saveTrigger(self):
        sched = self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )
        self.db.insert_test_data([
            fakedb.Object(id=self.SCHEDULERID, name='test', class_name='NightlyTriggerable'),
        ])

        sched.activate()

        _, d = sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                },
            ],
            set_props=None,
        )

        @d.addCallback
        def cb(_):
            self.db.state.assertState(
                self.SCHEDULERID,
                lastTrigger=[
                    [
                        {
                            "codebase": 'cb',
                            "revision": 'myrev',
                            "branch": 'br',
                            "project": 'p',
                            "repository": 'r',
                        },
                    ],
                    {},
                    None,
                    None,
                ],
            )

        return d

    def test_saveTrigger_noTrigger(self):
        sched = self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )
        self.db.insert_test_data([
            fakedb.Object(id=self.SCHEDULERID, name='test', class_name='NightlyTriggerable'),
        ])

        sched.activate()

        _, d = sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                },
            ],
            set_props=None,
        )

        self.clock.advance(60 * 60)  # Run for 1h

        @d.addCallback
        def cb(_):
            self.db.state.assertState(self.SCHEDULERID, lastTrigger=None)

        return d

    def test_triggerProperties(self):
        sched = self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )
        self.db.insert_test_data([
            fakedb.Object(id=self.SCHEDULERID, name='test', class_name='NightlyTriggerable'),
        ])

        sched.activate()

        sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                },
            ],
            properties.Properties(testprop='test'),
        )

        self.db.state.assertState(
            self.SCHEDULERID,
            lastTrigger=[
                [
                    {
                        "codebase": 'cb',
                        "revision": 'myrev',
                        "branch": 'br',
                        "project": 'p',
                        "repository": 'r',
                    },
                ],
                {'testprop': ['test', 'TEST']},
                None,
                None,
            ],
        )

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            properties={"testprop": ('test', 'TEST')},
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev',
                },
            ],
        )

    def test_savedProperties(self):
        sched = self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        value_json = (
            '[ [ {"codebase": "cb", "project": "p", "repository": "r", '
            '"branch": "br", "revision": "myrev"} ], '
            '{"testprop": ["test", "TEST"]}, null, null ]'
        )
        self.db.insert_test_data([
            fakedb.Object(id=self.SCHEDULERID, name='test', class_name='NightlyTriggerable'),
            fakedb.ObjectState(
                objectid=self.SCHEDULERID, name='lastTrigger', value_json=value_json
            ),
        ])

        sched.activate()

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            properties={'testprop': ('test', 'TEST')},
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev',
                },
            ],
        )

==> buildbot_4.2.1.orig.tar.gz: buildbot-4.2.1/master/buildbot/test/unit/schedulers/test_timed_NightlyTriggerable.py <==
# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members

from twisted.internet import defer
from twisted.internet import task
from twisted.trial import unittest

from buildbot.process import properties
from buildbot.schedulers import timed
from buildbot.test import fakedb
from buildbot.test.reactor import TestReactorMixin
from buildbot.test.util import scheduler
from buildbot.test.util.state import StateTestMixin


class NightlyTriggerable(
    scheduler.SchedulerMixin, TestReactorMixin, StateTestMixin, unittest.TestCase
):
    SCHEDULERID = 327
    OBJECTID = 1327

    @defer.inlineCallbacks
    def makeScheduler(self, firstBuildDuration=0, **kwargs):
        sched = yield self.attachScheduler(
            timed.NightlyTriggerable(**kwargs),
            self.OBJECTID,
            self.SCHEDULERID,
            overrideBuildsetMethods=True,
            createBuilderDB=True,
        )

        # add a Clock to help checking timing issues
        self.clock = sched._reactor = task.Clock()

        return sched

    @defer.inlineCallbacks
    def setUp(self):
        self.setup_test_reactor()
        yield self.setUpScheduler()

    # utilities

    def assertBuildsetAdded(self, sourcestamps=None, properties=None):
        if sourcestamps is None:
            sourcestamps = {}
        if properties is None:
            properties = {}
        properties['scheduler'] = ('test', 'Scheduler')
        self.assertEqual(
            self.addBuildsetCalls,
            [
                (
                    'addBuildsetForSourceStampsWithDefaults',
                    {
                        "builderNames": None,  # uses the default
                        "priority": None,
                        "properties": properties,
                        "reason": "The NightlyTriggerable scheduler named 'test' triggered this build",
                        "sourcestamps": sourcestamps,
                        "waited_for": False,
                    },
                ),
            ],
        )
        self.addBuildsetCalls = []

    def assertNoBuildsetAdded(self):
        self.assertEqual(self.addBuildsetCalls, [])

    # tests

    @defer.inlineCallbacks
    def test_constructor_no_reason(self):
        sched = yield self.makeScheduler(name='test', builderNames=['test'])
        self.assertEqual(
            sched.reason, "The NightlyTriggerable scheduler named 'test' triggered this build"
        )

    @defer.inlineCallbacks
    def test_constructor_reason(self):
        sched = yield self.makeScheduler(
            name='test', builderNames=['test'], reason="hourlytriggerable"
        )
        self.assertEqual(sched.reason, "hourlytriggerable")

    @defer.inlineCallbacks
    def test_constructor_month(self):
        sched = yield self.makeScheduler(name='test', builderNames=['test'], month='1')
        self.assertEqual(sched.month, "1")

    @defer.inlineCallbacks
    def test_timer_noBuilds(self):
        sched = yield self.makeScheduler(name='test', builderNames=['test'], minute=[5])

        sched.activate()
        self.clock.advance(60 * 60)  # Run for 1h

        self.assertEqual(self.addBuildsetCalls, [])

    @defer.inlineCallbacks
    def test_timer_oneTrigger(self):
        sched = yield self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        sched.activate()

        sched.trigger(
            False,
            [
                {
                    "revision": 'myrev',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "codebase": 'cb',
                },
            ],
            set_props=None,
        )

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev',
                },
            ]
        )

    @defer.inlineCallbacks
    def test_timer_twoTriggers(self):
        sched = yield self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        sched.activate()

        sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev1',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                }
            ],
            set_props=None,
        )
        sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev2',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                }
            ],
            set_props=None,
        )

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    # builds the second trigger's revision
                    "revision": 'myrev2',
                },
            ]
        )

    @defer.inlineCallbacks
    def test_timer_oneTrigger_then_noBuild(self):
        sched = yield self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        sched.activate()

        sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                }
            ],
            set_props=None,
        )

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev',
                },
            ]
        )

        self.clock.advance(60 * 60)  # Run for 1h

        # no trigger, so the second did not build
        self.assertNoBuildsetAdded()

    @defer.inlineCallbacks
    def test_timer_oneTriggers_then_oneTrigger(self):
        sched = yield self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        sched.activate()

        sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev1',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                }
            ],
            set_props=None,
        )

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev1',
                },
            ]
        )

        sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev2',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                }
            ],
            set_props=None,
        )

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev2',
                },
            ]
        )

    @defer.inlineCallbacks
    def test_savedTrigger(self):
        sched = yield self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        value_json = (
            '[ [ {"codebase": "cb", "project": "p", "repository": "r", '
            '"branch": "br", "revision": "myrev"} ], {}, null, null ]'
        )

        yield self.db.insert_test_data([
            fakedb.Object(id=self.SCHEDULERID, name='test', class_name='NightlyTriggerable'),
            fakedb.ObjectState(
                objectid=self.SCHEDULERID, name='lastTrigger', value_json=value_json
            ),
        ])

        sched.activate()

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev',
                },
            ]
        )

    @defer.inlineCallbacks
    def test_savedTrigger_dict(self):
        sched = yield self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        value_json = (
            '[ { "cb": {"codebase": "cb", "project": "p", "repository": "r", '
            '"branch": "br", "revision": "myrev"} }, {}, null, null ]'
        )
        yield self.db.insert_test_data([
            fakedb.Object(id=self.SCHEDULERID, name='test', class_name='NightlyTriggerable'),
            fakedb.ObjectState(
                objectid=self.SCHEDULERID, name='lastTrigger', value_json=value_json
            ),
        ])

        sched.activate()

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev',
                },
            ]
        )

    @defer.inlineCallbacks
    def test_saveTrigger(self):
        sched = yield self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )
        yield self.db.insert_test_data([
            fakedb.Object(id=self.SCHEDULERID, name='test', class_name='NightlyTriggerable'),
        ])

        sched.activate()

        _, d = sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                },
            ],
            set_props=None,
        )

        yield d
        yield self.assert_state(
            self.SCHEDULERID,
            lastTrigger=[
                [
                    {
                        "codebase": 'cb',
                        "revision": 'myrev',
                        "branch": 'br',
                        "project": 'p',
                        "repository": 'r',
                    },
                ],
                {},
                None,
                None,
            ],
        )

    @defer.inlineCallbacks
    def test_saveTrigger_noTrigger(self):
        sched = yield self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )
        yield self.db.insert_test_data([
            fakedb.Object(id=self.SCHEDULERID, name='test', class_name='NightlyTriggerable'),
        ])

        sched.activate()

        _, d = sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                },
            ],
            set_props=None,
        )

        self.clock.advance(60 * 60)  # Run for 1h

        yield d
        yield self.assert_state(self.SCHEDULERID, lastTrigger=None)

    @defer.inlineCallbacks
    def test_triggerProperties(self):
        sched = yield self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )
        yield self.db.insert_test_data([
            fakedb.Object(id=self.SCHEDULERID, name='test', class_name='NightlyTriggerable'),
        ])

        sched.activate()

        sched.trigger(
            False,
            [
                {
                    "codebase": 'cb',
                    "revision": 'myrev',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                },
            ],
            properties.Properties(testprop='test'),
        )

        yield self.assert_state(
            self.SCHEDULERID,
            lastTrigger=[
                [
                    {
                        "codebase": 'cb',
                        "revision": 'myrev',
                        "branch": 'br',
                        "project": 'p',
                        "repository": 'r',
                    },
                ],
                {'testprop': ['test', 'TEST']},
                None,
                None,
            ],
        )

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            properties={"testprop": ('test', 'TEST')},
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev',
                },
            ],
        )

    @defer.inlineCallbacks
    def test_savedProperties(self):
        sched = yield self.makeScheduler(
            name='test',
            builderNames=['test'],
            minute=[5],
            codebases={'cb': {'repository': 'annoying'}},
        )

        value_json = (
            '[ [ {"codebase": "cb", "project": "p", "repository": "r", '
            '"branch": "br", "revision": "myrev"} ], '
            '{"testprop": ["test", "TEST"]}, null, null ]'
        )
        yield self.db.insert_test_data([
            fakedb.Object(id=self.SCHEDULERID, name='test', class_name='NightlyTriggerable'),
            fakedb.ObjectState(
                objectid=self.SCHEDULERID, name='lastTrigger', value_json=value_json
            ),
        ])

        sched.activate()

        self.clock.advance(60 * 60)  # Run for 1h

        self.assertBuildsetAdded(
            properties={'testprop': ('test', 'TEST')},
            sourcestamps=[
                {
                    "codebase": 'cb',
                    "branch": 'br',
                    "project": 'p',
                    "repository": 'r',
                    "revision": 'myrev',
                },
            ],
        )
